#!/usr/bin/env bash



declare    tiles="fffffffgggggggggghhhhhpppppppppppaadddddjjjjjmmmsssssttt+"
declare    prefix="" IFS=$'\n' land water revd mapAlignment="Classic" temp zeroPad="0000" startPosOutput=""
declare -a lines=("") newLines=("") e00Layer=("") e01Layer=("") e02Layer=("") riverLayer=("") resLayer=("")
declare -i i j number mapWidth=100 mapHeight=50 halfMapHeight halfMapWidth count=0 startPosCount=0 oldLineWidth



# Function
GetTilesNumber() {
	let "number = ${RANDOM} % ${#tiles}"
}

usage="Usage:	$(basename ${0}) [-w width] [-h height] [-a alignment]\n\
	width is width in tiles. Default value: 100\n\
	height is height in tiles. Default value: 50\n\
	alignment may be either c for classic (the default) or i for isometric.\n\
	Map dimensions may vary slightly from specified values due to method.\n"

until [ -z "${1}" ] ; do
	case "${1}" in
		-w | -W | -x | -X | --width		)	shift ; mapWidth="${1}" ;;
		-h | -H | -y | -Y | --height		)	shift ; mapHeight="${1}" ;;
		-a | -A | --alignment			)	shift ; mapAlignment="${1}" ;;
		--help 					)	printf "${usage}" ; exit 1 ;;
		* 					)	echo -e "${usage}" ; exit 1 ;;
	esac
	shift
done

[ "${mapAlignment:0:1}" = "i" -o "${mapAlignment:0:1}" = "I" ] && mapAlignment="Isometric"

if [ ${mapWidth} -lt 16 -o ${mapHeight} -lt 16 ] ; then
	echo "FreeCiv minimum map width/height is 16"
	exit 1
fi



# Prepare to generate the map
halfMapHeight=$((${mapHeight}/2))
[ "${mapAlignment}" = "Isometric" ] && halfMapHeight=$((${mapHeight}/4))
halfMapWidth=$((${mapWidth}/2))

# "X" is used as a placeholder for land tiles which will be randomized later
# "water" is all shallow ocean (the space character)
land=$(temp=$(printf "%*s" ${halfMapWidth}) ; echo "${temp// /X}")
water=$(printf "%*s" ${halfMapWidth})

# Create other map layer arrays for output later
riverLayer[0]=$(temp=$(printf "%*s" ${mapWidth}) ; echo "${temp// /0}")
e00Layer[0]="${riverLayer[0]}"
e01Layer[0]="${riverLayer[0]}"
e02Layer[0]="${riverLayer[0]}"
resLayer[0]=$(printf "%*s" ${mapWidth})

for ((i=1;i<${mapHeight};i++)) ; do
	e00Layer[${i}]="${riverLayer[0]}"
	e01Layer[${i}]="${riverLayer[0]}"
	e02Layer[${i}]="${riverLayer[0]}"
	riverLayer[${i}]="${riverLayer[0]}"
	resLayer[${i}]="${resLayer[0]}"
done



# Create upper-left quarter of terrain (t) layer.
for ((i=0;i<${halfMapHeight};i++)) ; do
	if [ $((${i}%2)) -eq 0 ] ; then
		# Even numbered lines
		lines[${i}]="${prefix}${land:${#prefix}}"
		prefix="${prefix}X "
	else
		# Odd numbered lines
		lines[${i}]="${prefix}${water:${#prefix}}"
	fi

	# Mirror the left side of each line to the right side
	lines[${i}]="${lines[${i}]:0:${halfMapWidth}}"
	for ((j=${halfMapWidth};j>=0;j--)) ; do
		lines[${i}]="${lines[${i}]}${lines[${i}]:${j}:1}"
	done


done



# Repeat lines in reverse order for lower half of "t" map
for ((j=i,i=((${halfMapHeight}-1));i>=0;i--,j++)) ; do
	lines[${j}]="${lines[${i}]}"
done



# Isometric conversion
if [ "${mapAlignment}" = "Isometric" ] ; then

	for ((i=0,j=0;i<${#lines[*]};i++,j+=2)) ; do
		newLines[${j}]="${lines[${i}]}"
		newLines[$((${j}+1))]="${lines[${i}]}"
	done

	# leftover mess due to poor planning
	unset lines
	declare -a lines=("")
	lines=(${newLines[*]})
	unset newLines
	declare -a newLines
fi



# Randomize the land tiles (character X is the placeholder for land tiles)
for ((i=0;i<${#lines[*]};i++)) ; do
	for ((j=0;j<${#lines[${i}]};j++)) ; do
		# If tile is land
		if [ "${lines[${i}]:${j}:1}" = "X" ] ; then
			GetTilesNumber
			# If the preceeding chr of this line is the same as the new chr, pick again, one retry only.
			if [ "${newLines[${i}]:$((${j}-1)):1}" = "${tiles:${number}:1}" ] ; then
				GetTilesNumber
			fi
			newLines[${i}]="${newLines[${i}]}${tiles:${number}:1}"

			# If tile is -not- lake (+)
			if [ "${newLines[${i}]:${j}:1}" != "+" ] ; then
				# 1 in 10 chance of river on tile which is 10% of the land tiles
				if [ $(( ${RANDOM} % 10 )) -eq 0 -a $((${j}+1)) -le ${mapWidth} ] ; then
					riverLayer[${i}]="${riverLayer[${i}]:0:${j}}2${riverLayer[${i}]:$((${j}+1))}"

					# Fix for when the tile was the last one on the line
					[ ${#riverLayer[${i}]} -gt ${mapWidth} ] && riverLayer[${i}]="${riverLayer[${i}]:0:${mapWidth}}"
				fi

				# Generate random start positions, about 1 for every 50 land tiles or so.
				# Not recommending that as the number of default/aifill -players-,
				# just ensuring that there are enough startpos available from which the game can pick randomly
				if [ $(( ${RANDOM} % 50 )) -eq 0 ] ; then
					startPosOutput="${startPosOutput}${j},${i},FALSE,\"\"${IFS}"
					startPosCount=$(( ${startPosCount} + 1 ))
				fi
			fi
		else
			newLines[${i}]="${newLines[${i}]} "
		fi
	done
done




# Output the map for copy/paste to a FreeCiv ".sav" file

# t (terrain) layer
for ((i=0;i<${#newLines[*]};i++)) ; do
	echo "t${zeroPad:${#i}}${i}=\"${newLines[${i}]}\""
done

# Start positions list
echo "startpos_count=${startPosCount}"
echo "startpos={\"x\",\"y\",\"exclude\",\"nations\""
echo -n "${startPosOutput}"
echo '}'

# e00_ Irrigation, mine, pollution, oil wells layer
for ((i=0;i<${#newLines[*]};i++)) ; do
	echo "e00_${zeroPad:${#i}}${i}=\"${e00Layer[${i}]}\""
done

# e01_ Hut, farmland, fallout, fortress layer
for ((i=0;i<${#newLines[*]};i++)) ; do
	echo "e01_${zeroPad:${#i}}${i}=\"${e01Layer[${i}]}\""
done

# e02_ Rail, airbase, buoy, ruins layer
for ((i=0;i<${#newLines[*]};i++)) ; do
	echo "e02_${zeroPad:${#i}}${i}=\"${e02Layer[${i}]}\""
done


# e03_ River and road layer
for ((i=0;i<${#newLines[*]};i++)) ; do
	echo "e03_${zeroPad:${#i}}${i}=\"${riverLayer[${i}]}\""
done

# res (resources) layer
for ((i=0;i<${#newLines[*]};i++)) ; do
	echo "res${zeroPad:${#i}}${i}=\"${resLayer[${i}]}\""
done


exit 0
